import { assertNever } from "@fern-api/core-utils";
import { python } from ".";
import { AstNode } from "./core/AstNode";
export class Type extends AstNode {
    internalType;
    constructor(internalType) {
        super();
        this.internalType = internalType;
    }
    static int() {
        return new this({ type: "int" });
    }
    static float() {
        return new this({ type: "float" });
    }
    static bool() {
        return new this({ type: "bool" });
    }
    static str() {
        return new this({ type: "str" });
    }
    static bytes() {
        return new this({ type: "bytes" });
    }
    static list(value) {
        const listType = new this({ type: "list", value });
        listType.addReference(python.reference({ name: "List", modulePath: ["typing"] }));
        listType.inheritReferences(value);
        return listType;
    }
    static set(value) {
        const setType = new this({ type: "set", value });
        setType.addReference(python.reference({ name: "Set", modulePath: ["typing"] }));
        setType.inheritReferences(value);
        return setType;
    }
    static tuple(values) {
        const tupleType = new this({ type: "tuple", values });
        tupleType.addReference(python.reference({ name: "Tuple", modulePath: ["typing"] }));
        values.forEach((value) => tupleType.inheritReferences(value));
        return tupleType;
    }
    static dict(keyType, valueType) {
        const dictType = new this({ type: "dict", keyType, valueType });
        dictType.addReference(python.reference({ name: "Dict", modulePath: ["typing"] }));
        dictType.inheritReferences(keyType);
        dictType.inheritReferences(valueType);
        return dictType;
    }
    static none() {
        return new this({ type: "none" });
    }
    static uuid() {
        const uuidType = new this({ type: "uuid" });
        uuidType.addReference(python.reference({ name: "UUID", modulePath: ["uuid"] }));
        return uuidType;
    }
    static optional(value) {
        const optionalType = new this({ type: "optional", value });
        optionalType.addReference(python.reference({ name: "Optional", modulePath: ["typing"] }));
        optionalType.inheritReferences(value);
        return optionalType;
    }
    static union(values) {
        const unionType = new this({ type: "union", values });
        unionType.addReference(python.reference({ name: "Union", modulePath: ["typing"] }));
        values.forEach((value) => unionType.inheritReferences(value));
        return unionType;
    }
    static any() {
        const anyType = new this({ type: "any" });
        anyType.addReference(python.reference({ name: "Any", modulePath: ["typing"] }));
        return anyType;
    }
    static datetime() {
        const datetimeType = new this({ type: "datetime" });
        datetimeType.addReference(python.reference({ name: "datetime", modulePath: ["datetime"] }));
        return datetimeType;
    }
    static reference(value) {
        const referenceType = new this({ type: "reference", value });
        referenceType.addReference(value);
        return referenceType;
    }
    static literal(value) {
        const literalType = new this({ type: "literal", value });
        literalType.addReference(python.reference({ name: "Literal", modulePath: ["typing"] }));
        return literalType;
    }
    write(writer) {
        switch (this.internalType.type) {
            case "int":
                writer.write("int");
                break;
            case "float":
                writer.write("float");
                break;
            case "bool":
                writer.write("bool");
                break;
            case "str":
                writer.write("str");
                break;
            case "bytes":
                writer.write("bytes");
                break;
            case "list":
                writer.write("List[");
                this.internalType.value.write(writer);
                writer.write("]");
                break;
            case "set":
                writer.write("Set[");
                this.internalType.value.write(writer);
                writer.write("]");
                break;
            case "tuple":
                writer.write("Tuple[");
                this.internalType.values.forEach((value, index) => {
                    if (index > 0) {
                        writer.write(", ");
                    }
                    value.write(writer);
                });
                writer.write("]");
                break;
            case "dict":
                writer.write("Dict[");
                this.internalType.keyType.write(writer);
                writer.write(", ");
                this.internalType.valueType.write(writer);
                writer.write("]");
                break;
            case "none":
                writer.write("None");
                break;
            case "uuid":
                writer.write("UUID");
                break;
            case "optional":
                writer.write("Optional[");
                this.internalType.value.write(writer);
                writer.write("]");
                break;
            case "union":
                writer.write("Union[");
                this.internalType.values.forEach((value, index) => {
                    if (index > 0) {
                        writer.write(", ");
                    }
                    value.write(writer);
                });
                writer.write("]");
                break;
            case "any":
                writer.write("Any");
                break;
            case "reference":
                this.internalType.value.write(writer);
                break;
            case "datetime":
                writer.write("datetime");
                break;
            case "literal":
                if (typeof this.internalType.value === "string") {
                    writer.write(`Literal["${this.internalType.value}"]`);
                }
                else {
                    writer.write(`Literal[${this.internalType.value}]`);
                }
                break;
            default:
                assertNever(this.internalType);
        }
    }
}
