import { assertNever } from "@fern-api/core-utils";
import { AstNode } from "./core/AstNode";
import { Type } from "./Type";
import { Reference } from "./Reference";
export class TypeInstantiation extends AstNode {
    internalType;
    constructor(internalType) {
        super();
        this.internalType = internalType;
    }
    static int(value) {
        return new this({ type: "int", value });
    }
    static float(value) {
        return new this({ type: "float", value });
    }
    static bool(value) {
        return new this({ type: "bool", value });
    }
    static str(value, config = { multiline: false }) {
        return new this({ type: "str", value, config });
    }
    static bytes(value) {
        return new this({ type: "bytes", value });
    }
    static list(values) {
        const list = new this({ type: "list", values });
        values.forEach((value) => list.inheritReferences(value));
        return list;
    }
    static set(values) {
        const set = new this({ type: "set", values });
        values.forEach((value) => set.inheritReferences(value));
        return set;
    }
    static tuple(values) {
        const tuple = new this({ type: "tuple", values });
        values.forEach((value) => tuple.inheritReferences(value));
        return tuple;
    }
    static dict(entries) {
        const dict = new this({ type: "dict", entries });
        entries.forEach((entry) => {
            dict.inheritReferences(entry.key);
            dict.inheritReferences(entry.value);
        });
        return dict;
    }
    static none() {
        return new this({ type: "none" });
    }
    static uuid(value) {
        const uuid = new this({ type: "uuid", value });
        uuid.addReference(new Reference({ name: "UUID", modulePath: ["uuid"] }));
        return uuid;
    }
    write(writer) {
        switch (this.internalType.type) {
            case "int":
                writer.write(this.internalType.value.toString());
                break;
            case "float":
                writer.write(this.internalType.value.toString());
                break;
            case "bool":
                if (this.internalType.value) {
                    writer.write("True");
                }
                else {
                    writer.write("False");
                }
                break;
            case "str":
                if (this.internalType.config?.multiline) {
                    this.writeStringWithTripleQuotes({ writer, value: this.internalType.value });
                }
                else {
                    writer.write(`"${this.escapeString(this.internalType.value, { doubleQuote: true, newline: true })}"`);
                }
                break;
            case "bytes":
                writer.write(`b"${this.internalType.value}"`);
                break;
            case "list":
                writer.write("[");
                this.internalType.values.forEach((value, index) => {
                    if (index > 0) {
                        writer.write(", ");
                    }
                    value.write(writer);
                });
                writer.write("]");
                break;
            case "set":
                writer.write("{");
                this.internalType.values.forEach((value, index) => {
                    if (index > 0) {
                        writer.write(", ");
                    }
                    value.write(writer);
                });
                writer.write("}");
                break;
            case "tuple":
                writer.write("(");
                this.internalType.values.forEach((value, index) => {
                    if (index > 0) {
                        writer.write(", ");
                    }
                    value.write(writer);
                });
                writer.write(")");
                break;
            case "dict":
                writer.write("{");
                this.internalType.entries.forEach((entry, index) => {
                    if (index > 0) {
                        writer.write(", ");
                    }
                    entry.key.write(writer);
                    writer.write(": ");
                    entry.value.write(writer);
                });
                writer.write("}");
                break;
            case "none":
                writer.write("None");
                break;
            case "uuid":
                writer.write(`UUID("${this.internalType.value}")`);
                break;
            default:
                assertNever(this.internalType);
        }
    }
    writeStringWithTripleQuotes({ writer, value }) {
        writer.write('"""');
        const parts = value.split("\n");
        const head = parts[0] + "\n";
        const tail = parts.slice(1).join("\n");
        writer.write(this.escapeString(head));
        writer.writeNoIndent(this.escapeString(tail));
        writer.write('"""');
    }
    escapeString(value, config = { doubleQuote: true, newline: false }) {
        let escapedValue = value;
        if (config.doubleQuote) {
            escapedValue = escapedValue.replaceAll('"', '\\"');
        }
        if (config.newline) {
            escapedValue = escapedValue.replaceAll("\n", "\\n");
        }
        return escapedValue;
    }
}
export { Type };
